@@ -78,6 +78,9 @@ struct _VinoFBPrivate
   guint            update_timeout;
 
 #ifdef HAVE_XDAMAGE
+  GdkRegion       *pending_damage;
+  guint            damage_idle_handler;
+
   Damage           xdamage;
   int              xdamage_notify_event;
   XserverRegion    xdamage_region;
@@ -109,6 +112,8 @@ enum
 
 static void vino_fb_init_from_screen (VinoFB    *vfb,
 				      GdkScreen *screen);
+static void vino_fb_screen_size_changed (VinoFB    *vfb,
+					 GdkScreen *screen);
 
 #ifdef HAVE_XDAMAGE
 static GdkFilterReturn vino_fb_xdamage_event_filter (GdkXEvent *xevent,
@@ -153,7 +158,7 @@ vino_fb_get_image (VinoFB          *vfb,
   gdk_error_trap_push ();
 
 #ifdef HAVE_XSHM  
-  if (is_x_shm_segment && image->width == width && image->height && height)
+  if (is_x_shm_segment && image->width == width && image->height == height)
     {
       XShmGetImage (vfb->priv->xdisplay,
 		    xdrawable,
@@ -452,6 +457,14 @@ static void
 vino_fb_finalize_xdamage (VinoFB *vfb)
 {
 #ifdef HAVE_XDAMAGE
+  if (vfb->priv->damage_idle_handler)
+    g_source_remove (vfb->priv->damage_idle_handler);
+  vfb->priv->damage_idle_handler = 0;
+
+  if (vfb->priv->pending_damage)
+    gdk_region_destroy (vfb->priv->pending_damage);
+  vfb->priv->pending_damage = NULL;
+
   if (vfb->priv->fb_pixmap)
     XFreePixmap (vfb->priv->xdisplay, vfb->priv->fb_pixmap);
   vfb->priv->fb_pixmap = None;
@@ -518,6 +531,10 @@ vino_fb_finalize_screen_data (VinoFB *vfb)
 			   vfb->priv->fb_image_is_x_shm_segment,
 			   TRUE);
   vfb->priv->fb_image = NULL;
+  
+  g_signal_handlers_disconnect_by_func (vfb->priv->screen,
+					G_CALLBACK (vino_fb_screen_size_changed),
+					vfb);
 
 }
 
@@ -534,28 +551,33 @@ vino_fb_screen_size_changed (VinoFB    *vfb,
 }
 
 #ifdef HAVE_XDAMAGE
-static GdkFilterReturn
-vino_fb_xdamage_event_filter (GdkXEvent *xevent,
-			      GdkEvent  *event,
-			      VinoFB    *vfb)
+
+static gboolean
+vino_fb_xdamage_idle_handler (VinoFB *vfb)
 {
-  XEvent             *xev = (XEvent *) xevent;
-  XDamageNotifyEvent *notify;
-  GdkRectangle        damage;
-  int                 error;
+  GdkRectangle *damage = NULL;
+  XRectangle    xdamage;
+  int           n_rects;
+  int           error;
 
-  if (xev->type != vfb->priv->xdamage_notify_event)
-    return GDK_FILTER_CONTINUE;
+  g_assert (!gdk_region_empty (vfb->priv->pending_damage));
 
-  notify = (XDamageNotifyEvent *) xev;
+  gdk_region_get_rectangles (vfb->priv->pending_damage, &damage, &n_rects);
 
-  damage.x      = notify->area.x;
-  damage.y      = notify->area.y;
-  damage.width  = notify->area.width;
-  damage.height = notify->area.height;
+  xdamage.x      = damage->x;
+  xdamage.y      = damage->y;
+  xdamage.width  = damage->width;
+  xdamage.height = damage->height;
 
-  dprintf (POLLING, "Got DamageNotify event: %d %d %dx%d\n",
-	   damage.x, damage.y, damage.width, damage.height);
+  dprintf (POLLING, "Updating damaged region in idle: %d %d %dx%d\n",
+	   damage->x, damage->y, damage->width, damage->height);
+
+  /* subtract damage from server */
+  XFixesSetRegion (vfb->priv->xdisplay, vfb->priv->xdamage_region, &xdamage, 1);
+  XDamageSubtract (vfb->priv->xdisplay,
+		   vfb->priv->xdamage,
+		   vfb->priv->xdamage_region,
+		   None);
 
   gdk_error_trap_push ();
 
@@ -564,12 +586,12 @@ vino_fb_xdamage_event_filter (GdkXEvent *xevent,
 	     GDK_WINDOW_XWINDOW (vfb->priv->root_window),
 	     vfb->priv->fb_pixmap,
 	     vfb->priv->xdamage_copy_gc,
-	     damage.x,
-	     damage.y,
-	     damage.width,
-	     damage.height,
-	     damage.x,
-	     damage.y);
+	     damage->x,
+	     damage->y,
+	     damage->width,
+	     damage->height,
+	     damage->x,
+	     damage->y);
   XSync (vfb->priv->xdisplay, False);
 
   if ((error = gdk_error_trap_pop ()))
@@ -582,31 +604,73 @@ vino_fb_xdamage_event_filter (GdkXEvent *xevent,
       g_warning ("Received a '%s' X Window System error while copying damaged pixels",
 		 error_text);
       g_warning ("Failed image = %d, %d %dx%d - screen = %dx%d",
-		 damage.x,
-		 damage.y,
-		 damage.width,
-		 damage.height,
+		 damage->x,
+		 damage->y,
+		 damage->width,
+		 damage->height,
 		 gdk_screen_get_width (vfb->priv->screen),
 		 gdk_screen_get_height (vfb->priv->screen));
 #endif
-      return GDK_FILTER_REMOVE;
+      goto out;
     }
 
   /* add damage to our region */
   if (vfb->priv->damage_region)
-    gdk_region_union_with_rect (vfb->priv->damage_region, &damage);
+    gdk_region_union_with_rect (vfb->priv->damage_region, damage);
   else
-    vfb->priv->damage_region = gdk_region_rectangle (&damage);
-
-  /* subtract damage from server */
-  XFixesSetRegion (vfb->priv->xdisplay, vfb->priv->xdamage_region, &notify->area, 1);
-  XDamageSubtract (vfb->priv->xdisplay,
-		   vfb->priv->xdamage,
-		   vfb->priv->xdamage_region,
-		   None);
+    vfb->priv->damage_region = gdk_region_rectangle (damage);
 
   emit_damage_notify (vfb);
 
+ out:
+  {
+    GdkRegion *tmp;
+
+    tmp = gdk_region_rectangle (damage);
+    gdk_region_subtract (vfb->priv->pending_damage, tmp);
+    gdk_region_destroy (tmp);
+  }
+
+  g_free (damage);
+
+  if (gdk_region_empty (vfb->priv->pending_damage))
+    {
+      vfb->priv->damage_idle_handler = 0;
+      return FALSE;
+    }
+
+  return TRUE;
+}
+
+static GdkFilterReturn
+vino_fb_xdamage_event_filter (GdkXEvent *xevent,
+			      GdkEvent  *event,
+			      VinoFB    *vfb)
+{
+  XEvent             *xev = (XEvent *) xevent;
+  XDamageNotifyEvent *notify;
+  GdkRectangle        damage;
+
+  if (xev->type != vfb->priv->xdamage_notify_event)
+    return GDK_FILTER_CONTINUE;
+
+  notify = (XDamageNotifyEvent *) xev;
+
+  damage.x      = notify->area.x;
+  damage.y      = notify->area.y;
+  damage.width  = notify->area.width;
+  damage.height = notify->area.height;
+
+  dprintf (POLLING, "Got DamageNotify event: %d %d %dx%d, more = %s, level = %d\n",
+	   damage.x, damage.y, damage.width, damage.height,
+	   notify->more ? "(true)" : "(false)", notify->level);
+
+  gdk_region_union_with_rect (vfb->priv->pending_damage, &damage);
+
+  if (!vfb->priv->damage_idle_handler)
+    vfb->priv->damage_idle_handler =
+      g_idle_add ((GSourceFunc) vino_fb_xdamage_idle_handler, vfb);
+
   return GDK_FILTER_REMOVE;
 }
 #endif /* HAVE_XDAMAGE */
@@ -629,7 +693,7 @@ vino_fb_init_xdamage (VinoFB *vfb)
 
   vfb->priv->xdamage = XDamageCreate (vfb->priv->xdisplay,
 				      GDK_WINDOW_XWINDOW (vfb->priv->root_window),
-				      XDamageReportRawRectangles);
+				      XDamageReportDeltaRectangles);
   if (vfb->priv->xdamage == None)
     return;
 
@@ -654,6 +718,8 @@ vino_fb_init_xdamage (VinoFB *vfb)
 			 (GdkFilterFunc) vino_fb_xdamage_event_filter,
 			 vfb);
 
+  vfb->priv->pending_damage = gdk_region_new ();
+
   vfb->priv->use_xdamage = TRUE;
 #endif
 }
